@@ -1,6 +1,6 @@
-#include <ScriptingBridge/ScriptingBridge.h>
 #include <Carbon/Carbon.h>
 #include <Cocoa/Cocoa.h>
+#include <objc/objc-runtime.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -16,13 +16,22 @@
 #include <unistd.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/un.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <netdb.h>
+#include <poll.h>
 #include <semaphore.h>
 #include <pthread.h>
+#include <pwd.h>
 
 #include "misc/extern.h"
 #include "misc/macros.h"
 #include "misc/memory_pool.h"
 #include "misc/ts.h"
+//#include "misc/autorelease.h"
 #include "misc/notify.h"
 #include "misc/log.h"
 #include "misc/helpers.h"
@@ -30,13 +39,10 @@
 #define HASHTABLE_IMPLEMENTATION
 #include "misc/hashtable.h"
 #undef HASHTABLE_IMPLEMENTATION
-#include "misc/socket.h"
-#include "misc/socket.c"
 
-#include "osax/sa.h"
-#include "osax/mach_loader.c"
-#include "osax/sa.m"
+#include "osax/common.h"
 
+#include "sa.h"
 #include "event.h"
 #include "event_loop.h"
 #include "event_signal.h"
@@ -56,6 +62,8 @@
 #include "window_manager.h"
 #include "mouse.h"
 
+#include "sa.m"
+#include "mission_control.c"
 #include "event_loop.c"
 #include "event.c"
 #include "event_signal.c"
@@ -74,5 +82,4 @@
 #include "space_manager.c"
 #include "window_manager.c"
 #include "mouse.c"
-
 #include "yabai.c"
