@@ -1,7 +1,16 @@
 #ifndef HELPERS_H
 #define HELPERS_H
 
-const CFStringRef kAXEnhancedUserInterface = CFSTR("AXEnhancedUserInterface");
+struct rgba_color
+{
+    uint32_t p;
+    float r;
+    float g;
+    float b;
+    float a;
+};
+
+static const CFStringRef kAXEnhancedUserInterface = CFSTR("AXEnhancedUserInterface");
 
 static const char *bool_str[] = { "off", "on" };
 
@@ -13,24 +22,69 @@ static const char *layer_str[] =
     [LAYER_ABOVE] = "above"
 };
 
-struct signal_args
+static inline float ease_out_cubic(float t)
 {
-    char name[2][255];
-    char value[2][255];
-    void *entity;
-    void *param1;
-};
+    return 1.0f - powf(1.0f - t, 3.0f);
+}
 
-struct rgba_color
+#define ANIMATE(animation_connection, animation_duration, easing_function, code_block) \
+{                                                                                      \
+    int frame_duration = 4;                                                            \
+    int total_duration = (int)(animation_duration * 1000.0f);                          \
+    int frame_count = (int)(((float) total_duration / (float) frame_duration) + 1.0f); \
+                                                                                       \
+    for (int frame_index = 1; frame_index <= frame_count; ++frame_index) {             \
+        float t = (float) frame_index / (float) frame_count;                           \
+        if (t < 0.0f) t = 0.0f;                                                        \
+        if (t > 1.0f) t = 1.0f;                                                        \
+                                                                                       \
+        float mt = easing_function(t);                                                 \
+        CFTypeRef transaction = SLSTransactionCreate(animation_connection);            \
+                                                                                       \
+        code_block                                                                     \
+                                                                                       \
+        SLSTransactionCommit(transaction, 0);                                          \
+        CFRelease(transaction);                                                        \
+                                                                                       \
+        usleep(frame_duration*1000);                                                   \
+    }                                                                                  \
+}
+
+static inline bool socket_open(int *sockfd)
 {
-    uint32_t p;
-    float r;
-    float g;
-    float b;
-    float a;
-};
+    *sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
+    return *sockfd != -1;
+}
 
-struct rgba_color rgba_color_from_hex(uint32_t color)
+static inline bool socket_connect(int sockfd, char *socket_path)
+{
+    struct sockaddr_un socket_address;
+    socket_address.sun_family = AF_UNIX;
+
+    snprintf(socket_address.sun_path, sizeof(socket_address.sun_path), "%s", socket_path);
+    return connect(sockfd, (struct sockaddr *) &socket_address, sizeof(socket_address)) != -1;
+}
+
+static inline void socket_close(int sockfd)
+{
+    shutdown(sockfd, SHUT_RDWR);
+    close(sockfd);
+}
+
+static inline char *json_optional_bool(int value)
+{
+    if (value == 0) return "null";
+    if (value == 1) return "true";
+
+    return "false";
+}
+
+static inline char *json_bool(bool value)
+{
+    return value ? "true" : "false";
+}
+
+static inline struct rgba_color rgba_color_from_hex(uint32_t color)
 {
     struct rgba_color result;
     result.p = color;
@@ -110,29 +164,30 @@ static inline char *ts_string_escape(char *s)
     char *cursor = _string_escape_pre(s, &size_in_bytes);
     if (!cursor) return NULL;
 
-    char *result = ts_alloc(sizeof(char) * (size_in_bytes+1));
+    char *result = ts_alloc_unaligned(sizeof(char) * (size_in_bytes+1));
     result[size_in_bytes] = '\0';
     _string_escape_post(s, cursor, result);
 
     return result;
 }
 
-/*
-static inline char *string_escape(char *s)
+static inline CFNumberRef CFNUM32(int32_t num)
 {
-    int size_in_bytes;
-    char *cursor = _string_escape_pre(s, &size_in_bytes);
-    if (!cursor) return NULL;
-
-    char *result = malloc(sizeof(char) * (size_in_bytes+1));
-    result[size_in_bytes] = '\0';
-    _string_escape_post(s, cursor, result);
-
-    return result;
+    return CFNumberCreate(NULL, kCFNumberSInt32Type, &num);
 }
-*/
 
-static CFArrayRef cfarray_of_cfnumbers(void *values, size_t size, int count, CFNumberType type)
+static inline void sls_window_disable_shadow(uint32_t id)
+{
+    CFNumberRef density = CFNUM32(0);
+    const void *keys[1] = { CFSTR("com.apple.WindowShadowDensity") };
+    const void *values[1] = { density };
+    CFDictionaryRef options = CFDictionaryCreate(NULL, keys, values, 1, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
+    SLSWindowSetShadowProperties(id, options);
+    CFRelease(density);
+    CFRelease(options);
+}
+
+static inline CFArrayRef cfarray_of_cfnumbers(void *values, size_t size, int count, CFNumberType type)
 {
     CFNumberRef temp[count];
 
@@ -152,8 +207,7 @@ static CFArrayRef cfarray_of_cfnumbers(void *values, size_t size, int count, CFN
 static inline char *ts_cfstring_copy(CFStringRef string)
 {
     CFIndex num_bytes = CFStringGetMaximumSizeForEncoding(CFStringGetLength(string), kCFStringEncodingUTF8);
-    char *result = ts_alloc(num_bytes + 1);
-    if (!result) return NULL;
+    char *result = ts_alloc_unaligned(num_bytes + 1);
 
     if (!CFStringGetCString(string, result, num_bytes + 1, kCFStringEncodingUTF8)) {
         result = NULL;
@@ -179,8 +233,7 @@ static inline char *cfstring_copy(CFStringRef string)
 static inline char *ts_string_copy(char *s)
 {
     int length = strlen(s);
-    char *result = ts_alloc(length + 1);
-    if (!result) return NULL;
+    char *result = ts_alloc_unaligned(length + 1);
 
     memcpy(result, s, length);
     result[length] = '\0';
@@ -229,22 +282,7 @@ static inline bool ensure_executable_permission(char *filename)
     return true;
 }
 
-static bool fork_exec(char *command, struct signal_args *args)
-{
-    int pid = fork();
-    if (pid == -1) return false;
-    if (pid !=  0) return true;
-
-    if (args) {
-        if (*args->name[0]) setenv(args->name[0], args->value[0], 1);
-        if (*args->name[1]) setenv(args->name[1], args->value[1], 1);
-    }
-
-    char *exec[] = { "/usr/bin/env", "sh", "-c", command, NULL};
-    exit(execvp(exec[0], exec));
-}
-
-static bool ax_privilege(void)
+static inline bool ax_privilege(void)
 {
     const void *keys[] = { kAXTrustedCheckOptionPrompt };
     const void *values[] = { kCFBooleanTrue };
@@ -297,20 +335,29 @@ static inline bool psn_equals(ProcessSerialNumber *a, ProcessSerialNumber *b)
 }
 #pragma clang diagnostic pop
 
-static inline int euclidean_distance(CGPoint p1, CGPoint p2)
+static inline float cgrect_clamp_x_radius(CGRect frame, float radius)
 {
-    int dx = p1.x - p2.x;
-    int dy = p1.y - p2.y;
-    return dx*dx + dy*dy;
+    if (radius * 2 > CGRectGetWidth(frame)) {
+        radius = CGRectGetWidth(frame) / 2;
+    }
+    return radius;
 }
 
-static bool cgrect_contains_point(CGRect r, CGPoint p)
+static inline float cgrect_clamp_y_radius(CGRect frame, float radius)
+{
+    if (radius * 2 > CGRectGetHeight(frame)) {
+        radius = CGRectGetHeight(frame) / 2;
+    }
+    return radius;
+}
+
+static inline bool cgrect_contains_point(CGRect r, CGPoint p)
 {
     return p.x >= r.origin.x && p.x <= r.origin.x + r.size.width &&
            p.y >= r.origin.y && p.y <= r.origin.y + r.size.height;
 }
 
-static bool triangle_contains_point(CGPoint t[3], CGPoint p)
+static inline bool triangle_contains_point(CGPoint t[3], CGPoint p)
 {
     float l1 = (p.x - t[0].x) * (t[2].y - t[0].y) - (t[2].x - t[0].x) * (p.y - t[0].y);
     float l2 = (p.x - t[1].x) * (t[0].y - t[1].y) - (t[0].x - t[1].x) * (p.y - t[1].y);
@@ -319,7 +366,7 @@ static bool triangle_contains_point(CGPoint t[3], CGPoint p)
     return (l1 > 0.0f && l2 > 0.0f && l3 > 0.0f) || (l1 < 0.0f && l2 < 0.0f && l3 < 0.0f);
 }
 
-static int regex_match(bool valid, regex_t *regex, const char *match)
+static inline int regex_match(bool valid, regex_t *regex, const char *match)
 {
     if (!valid) return REGEX_MATCH_UD;
 
