@@ -1,5 +1,3 @@
-#include "process_manager.h"
-
 extern struct event_loop g_event_loop;
 extern void *g_workspace_context;
 
@@ -63,6 +61,31 @@ static bool process_is_observable(struct process *process)
         return false;
     }
 
+    if (string_equals(process->name, "callservicesd")) {
+        debug("%s: %s (%d) is blacklisted! ignoring..\n", __FUNCTION__, process->name, process->pid);
+        return false;
+    }
+
+    if (string_equals(process->name, "UIKitSystem")) {
+        debug("%s: %s (%d) is blacklisted! ignoring..\n", __FUNCTION__, process->name, process->pid);
+        return false;
+    }
+
+    if (string_equals(process->name, "imklaunchagent")) {
+        debug("%s: %s (%d) is blacklisted! ignoring..\n", __FUNCTION__, process->name, process->pid);
+        return false;
+    }
+
+    if (string_equals(process->name, "LinkedNotesUIService")) {
+        debug("%s: %s (%d) is blacklisted! ignoring..\n", __FUNCTION__, process->name, process->pid);
+        return false;
+    }
+
+    if (string_equals(process->name, "Universal Control")) {
+        debug("%s: %s (%d) is blacklisted! ignoring..\n", __FUNCTION__, process->name, process->pid);
+        return false;
+    }
+
     if (string_equals(process->name, "Dock")) {
         debug("%s: %s (%d) is blacklisted! ignoring..\n", __FUNCTION__, process->name, process->pid);
         return false;
@@ -110,6 +133,8 @@ static PROCESS_EVENT_HANDLER(process_handler)
         process->terminated = true;
         process_manager_remove_process(pm, &psn);
 
+        __asm__ __volatile__ ("" ::: "memory");
+
         event_loop_post(&g_event_loop, APPLICATION_TERMINATED, process, 0, NULL);
     } break;
     case kEventAppFrontSwitched: {
@@ -194,7 +219,10 @@ void process_manager_init(struct process_manager *pm)
     pm->type[2].eventClass = kEventClassApplication;
     pm->type[2].eventKind  = kEventAppFrontSwitched;
     table_init(&pm->process, 125, hash_psn, compare_psn);
+
+    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     process_manager_add_running_processes(pm);
+    [pool drain];
 }
 
 #pragma clang diagnostic push
@@ -205,6 +233,7 @@ bool process_manager_begin(struct process_manager *pm)
     _SLPSGetFrontProcess(&front_psn);
     GetProcessPID(&front_psn, &g_process_manager.front_pid);
     g_process_manager.last_front_pid = g_process_manager.front_pid;
+    g_process_manager.switch_event_time = GetCurrentEventTime();
     return InstallEventHandler(pm->target, pm->handler, 3, pm->type, pm, &pm->ref) == noErr;
 }
 #pragma clang diagnostic pop
