@@ -1,7 +1,5 @@
-#include "space_manager.h"
-
 extern struct window_manager g_window_manager;
-extern bool g_mission_control_active;
+extern int g_mission_control_active;
 extern int g_connection;
 
 static TABLE_HASH_FUNC(hash_view)
@@ -19,9 +17,9 @@ bool space_manager_has_separate_spaces(void)
     return SLSGetSpaceManagementMode(g_connection) == 1;
 }
 
-bool space_manager_query_active_space(FILE *rsp)
+bool space_manager_query_space(FILE *rsp, uint64_t sid)
 {
-    struct view *view = space_manager_query_view(&g_space_manager, space_manager_active_space());
+    struct view *view = space_manager_query_view(&g_space_manager, sid);
     if (!view) return false;
 
     view_serialize(rsp, view);
@@ -69,7 +67,7 @@ bool space_manager_query_spaces_for_display(FILE *rsp, uint32_t did)
 
 bool space_manager_query_spaces_for_displays(FILE *rsp)
 {
-    uint32_t display_count;
+    int display_count;
     uint32_t *display_list = display_manager_active_display_list(&display_count);
     if (!display_list) return false;
 
@@ -127,22 +125,16 @@ void space_manager_mark_view_invalid(struct space_manager *sm,  uint64_t sid)
     view->is_valid = false;
 }
 
-void space_manager_mark_view_dirty(struct space_manager *sm,  uint64_t sid)
-{
-    struct view *view = space_manager_find_view(sm, sid);
-    if (view->layout == VIEW_FLOAT) return;
-
-    view->is_dirty = true;
-}
-
 void space_manager_untile_window(struct space_manager *sm, struct view *view, struct window *window)
 {
     if (view->layout == VIEW_FLOAT) return;
 
-    view_remove_window_node(view, window);
-    view_flush(view);
+    struct window_node *node = view_remove_window_node(view, window);
+    if (!node) return;
 
-    if (!space_is_visible(view->sid)) {
+    if (space_is_visible(view->sid)) {
+        window_node_flush(node);
+    } else {
         view->is_dirty = true;
     }
 }
@@ -381,12 +373,12 @@ bool space_manager_mirror_space(struct space_manager *sm, uint64_t sid, enum win
     return true;
 }
 
-bool space_manager_balance_space(struct space_manager *sm, uint64_t sid)
+bool space_manager_balance_space(struct space_manager *sm, uint64_t sid, uint32_t axis_flag)
 {
     struct view *view = space_manager_find_view(sm, sid);
     if (view->layout != VIEW_BSP) return false;
 
-    window_node_equalize(view->root);
+    window_node_equalize(view->root, axis_flag);
     view_update(view);
     view_flush(view);
 
@@ -395,27 +387,18 @@ bool space_manager_balance_space(struct space_manager *sm, uint64_t sid)
 
 struct view *space_manager_tile_window_on_space_with_insertion_point(struct space_manager *sm, struct window *window, uint64_t sid, uint32_t insertion_point)
 {
-    uint32_t prev_insertion_point;
-
     struct view *view = space_manager_find_view(sm, sid);
     if (view->layout == VIEW_FLOAT) return view;
 
-    if (view->layout == VIEW_BSP && insertion_point) {
-        prev_insertion_point = view->insertion_point;
-        view->insertion_point = insertion_point;
-    }
+    struct window_node *node = view_add_window_node_with_insertion_point(view, window, insertion_point);
+    assert(node);
 
-    view_add_window_node(view, window);
-    view_flush(view);
-
-    if (!space_is_visible(view->sid)) {
+    if (space_is_visible(view->sid)) {
+        window_node_flush(node);
+    } else {
         view->is_dirty = true;
     }
 
-    if (view->layout == VIEW_BSP && insertion_point) {
-        view->insertion_point = prev_insertion_point;
-    }
-
     return view;
 }
 
@@ -434,7 +417,7 @@ void space_manager_toggle_window_split(struct space_manager *sm, struct window *
         node->parent->split = node->parent->split == SPLIT_Y ? SPLIT_X : SPLIT_Y;
 
         if (sm->auto_balance) {
-            window_node_equalize(view->root);
+            window_node_equalize(view->root, SPLIT_X | SPLIT_Y);
             view_update(view);
             view_flush(view);
         } else {
@@ -613,7 +596,10 @@ uint64_t space_manager_active_space(void)
 
 void space_manager_move_window_to_space(uint64_t sid, struct window *window)
 {
-    CFArrayRef window_list_ref = cfarray_of_cfnumbers(&window->id, sizeof(uint32_t), 1, kCFNumberSInt32Type);
+    int window_count = window->border.id ? 2 : 1;
+    uint32_t window_list[2] = { window->id, window->border.id };
+
+    CFArrayRef window_list_ref = cfarray_of_cfnumbers(window_list, sizeof(uint32_t), window_count, kCFNumberSInt32Type);
     SLSMoveWindowsToManagedSpace(g_connection, window_list_ref, sid);
     CFRelease(window_list_ref);
 }
@@ -634,7 +620,9 @@ enum space_op_error space_manager_focus_space(uint64_t sid)
     if (is_animating) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
 
     if (scripting_addition_focus_space(sid)) {
-        if (focus_display) display_manager_focus_display(new_did);
+        if (focus_display) {
+            display_manager_focus_display(new_did, sid);
+        }
     } else {
         return SPACE_OP_ERROR_SCRIPTING_ADDITION;
     }
@@ -642,6 +630,23 @@ enum space_op_error space_manager_focus_space(uint64_t sid)
     return SPACE_OP_ERROR_SUCCESS;
 }
 
+static inline uint64_t space_manager_find_first_user_space_for_display(uint32_t did)
+{
+    int count;
+    uint64_t *space_list = display_space_list(did, &count);
+    if (!space_list) return 0;
+
+    for (int i = 0; i < count; ++i) {
+        uint64_t sid = space_list[i];
+
+        if (space_is_user(sid)) {
+            return sid;
+        }
+    }
+
+    return 0;
+}
+
 static inline bool space_manager_is_space_last_user_space(uint64_t sid)
 {
     bool result = true;
@@ -792,10 +797,20 @@ enum space_op_error space_manager_destroy_space(uint64_t sid)
     if (!space_is_user(sid)) return SPACE_OP_ERROR_INVALID_TYPE;
     if (space_manager_is_space_last_user_space(sid)) return SPACE_OP_ERROR_INVALID_SRC;
 
-    bool is_animating = display_manager_display_is_animating(space_display_id(sid));
+    uint32_t did = space_display_id(sid);
+    uint64_t first_sid = space_manager_find_first_user_space_for_display(did);
+
+    bool is_animating = display_manager_display_is_animating(did);
     if (is_animating) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
 
-    return scripting_addition_destroy_space(sid) ? SPACE_OP_ERROR_SUCCESS : SPACE_OP_ERROR_SCRIPTING_ADDITION;
+    bool success = scripting_addition_destroy_space(sid);
+    if (!success) return SPACE_OP_ERROR_SCRIPTING_ADDITION;
+
+    if (first_sid) {
+        window_manager_validate_and_check_for_windows_on_space(&g_space_manager, &g_window_manager, first_sid);
+    }
+
+    return SPACE_OP_ERROR_SUCCESS;
 }
 
 enum space_op_error space_manager_add_space(uint64_t sid)
@@ -860,7 +875,7 @@ void space_manager_mark_spaces_invalid_for_display(struct space_manager *sm, uin
 
 void space_manager_mark_spaces_invalid(struct space_manager *sm)
 {
-    uint32_t display_count = 0;
+    int display_count;
     uint32_t *display_list = display_manager_active_display_list(&display_count);
     if (!display_list) return;
 
@@ -871,16 +886,14 @@ void space_manager_mark_spaces_invalid(struct space_manager *sm)
 
 bool space_manager_refresh_application_windows(struct space_manager *sm)
 {
+    int refresh_count = buf_len(g_window_manager.applications_to_refresh);
+    if (!refresh_count) return false;
+
     int window_count = g_window_manager.window.count;
-    for (int i = 0; i < g_window_manager.application.capacity; ++i) {
-        struct bucket *bucket = g_window_manager.application.buckets[i];
-        while (bucket) {
-            if (bucket->value) {
-                struct application *application = bucket->value;
-                window_manager_add_application_windows(sm, &g_window_manager, application);
-            }
-            bucket = bucket->next;
-        }
+    for (int i = 0; i < refresh_count; ++i) {
+        struct application *application = g_window_manager.applications_to_refresh[i];
+        debug("%s: %s has windows that are not yet resolved\n", __FUNCTION__, application->name);
+        window_manager_add_existing_application_windows(sm, &g_window_manager, application, i);
     }
 
     return window_count != g_window_manager.window.count;
@@ -911,7 +924,7 @@ void space_manager_handle_display_add(struct space_manager *sm, uint32_t did)
 
     for (int i = 0; i < space_count; ++i) {
         uint64_t sid = space_list[i];
-        CFStringRef uuid = space_uuid(sid);
+        CFStringRef uuid = SLSSpaceCopyName(g_connection, sid);
         if (!uuid) continue;
 
         for (int j = 0; j < list_count; ++j) {
@@ -950,12 +963,13 @@ void space_manager_init(struct space_manager *sm)
     sm->layout = VIEW_FLOAT;
     sm->split_ratio = 0.5f;
     sm->auto_balance = false;
+    sm->split_type = SPLIT_AUTO;
     sm->window_placement = CHILD_SECOND;
     sm->labels = NULL;
 
     table_init(&sm->view, 23, hash_view, compare_view);
 
-    uint32_t display_count;
+    int display_count;
     uint32_t *display_list = display_manager_active_display_list(&display_count);
     if (!display_list) return;
 
