@@ -1,5 +1,3 @@
-#include "view.h"
-
 extern int g_connection;
 extern int g_floating_window_level;
 extern struct display_manager g_display_manager;
@@ -9,19 +7,19 @@ extern struct window_manager g_window_manager;
 void insert_feedback_show(struct window_node *node)
 {
     CFTypeRef frame_region;
-    CGRect frame = {{(int)node->area.x, (int)node->area.y},{(int)(node->area.w+0.5f), (int)(node->area.h+0.5f)}};
+    CGRect frame = {{node->area.x, node->area.y},{node->area.w, node->area.h}};
     CGSNewRegionWithRect(&frame, &frame_region);
 
     if (!node->feedback_window.id) {
-        uint64_t tags = kCGSIgnoreForExposeTagBit | kCGSIgnoreForEventsTagBit | kCGSDisableShadowTagBit;
+        uint64_t tag = 1ULL << 46;
         SLSNewWindow(g_connection, 2, 0, 0, frame_region, &node->feedback_window.id);
-        SLSSetWindowTags(g_connection, node->feedback_window.id, &tags, 64);
-        SLSSetWindowResolution(g_connection, node->feedback_window.id, 1.0f);
+        SLSSetWindowTags(g_connection, node->feedback_window.id, &tag, 64);
+        sls_window_disable_shadow(node->feedback_window.id);
+        SLSSetWindowResolution(g_connection, node->feedback_window.id, 2.0f);
         SLSSetWindowOpacity(g_connection, node->feedback_window.id, 0);
         SLSSetWindowLevel(g_connection, node->feedback_window.id, g_floating_window_level);
         node->feedback_window.context = SLWindowContextCreate(g_connection, node->feedback_window.id, 0);
-        int width = g_window_manager.enable_window_border ? g_window_manager.border_width : 2;
-        CGContextSetLineWidth(node->feedback_window.context, width);
+        CGContextSetLineWidth(node->feedback_window.context, g_window_manager.border_width);
         CGContextSetRGBFillColor(node->feedback_window.context,
                                    g_window_manager.insert_feedback_color.r,
                                    g_window_manager.insert_feedback_color.g,
@@ -36,61 +34,61 @@ void insert_feedback_show(struct window_node *node)
     }
 
     frame.origin.x = 0; frame.origin.y = 0;
-    CGFloat x1, y1, x2, y2, x3, y3, x4, y4;
-    CGFloat minx = CGRectGetMinX(frame), midx = CGRectGetMidX(frame), maxx = CGRectGetMaxX(frame);
-    CGFloat miny = CGRectGetMinY(frame), midy = CGRectGetMidY(frame), maxy = CGRectGetMaxY(frame);
+    CGFloat clip_x, clip_y, clip_w, clip_h;
+    CGFloat midx = CGRectGetMidX(frame);
+    CGFloat midy = CGRectGetMidY(frame);
 
     switch (node->insert_dir) {
     case DIR_NORTH: {
-        x1 = minx; y1 = midy;
-        x2 = minx; y2 = maxy;
-        x3 = maxx; y3 = maxy;
-        x4 = maxx; y4 = midy;
+        clip_x = -0.5f * g_window_manager.border_width;
+        clip_y = midy - 0.5f * g_window_manager.border_width;
+        clip_w = g_window_manager.border_width;
+        clip_h = g_window_manager.border_width;
     } break;
     case DIR_EAST: {
-        x1 = midx; y1 = miny;
-        x2 = maxx; y2 = miny;
-        x3 = maxx; y3 = maxy;
-        x4 = midx; y4 = maxy;
+        clip_x = midx - 0.5f * g_window_manager.border_width;
+        clip_y = -0.5f * g_window_manager.border_width;
+        clip_w = g_window_manager.border_width;
+        clip_h = g_window_manager.border_width;
     } break;
     case DIR_SOUTH: {
-        x1 = minx; y1 = midy;
-        x2 = minx; y2 = miny;
-        x3 = maxx; y3 = miny;
-        x4 = maxx; y4 = midy;
+        clip_x = -0.5f * g_window_manager.border_width;
+        clip_y = -0.5f * g_window_manager.border_width;
+        clip_w = g_window_manager.border_width;
+        clip_h = -midy + g_window_manager.border_width;
     } break;
     case DIR_WEST: {
-        x1 = midx; y1 = miny;
-        x2 = minx; y2 = miny;
-        x3 = minx; y3 = maxy;
-        x4 = midx; y4 = maxy;
+        clip_x = -0.5f * g_window_manager.border_width;
+        clip_y = -0.5f * g_window_manager.border_width;
+        clip_w = -midx + g_window_manager.border_width;
+        clip_h = g_window_manager.border_width;
     } break;
     case STACK: {
-        x1 = minx; y1 = miny;
-        x2 = minx; y2 = maxy;
-        x3 = maxx; y3 = maxy;
-        x4 = maxx; y4 = miny;
+        clip_x = -0.5f * g_window_manager.border_width;
+        clip_y = -0.5f * g_window_manager.border_width;
+        clip_w = g_window_manager.border_width;
+        clip_h = g_window_manager.border_width;
     } break;
     }
 
-    CGRect fill = { {x1, y1}, { x3 - x1, y3 - y1 } };
-    CGMutablePathRef outline = CGPathCreateMutable();
-    CGPathMoveToPoint(outline, NULL, x1, y1);
-    CGPathAddLineToPoint(outline, NULL, x2, y2);
-    CGPathAddLineToPoint(outline, NULL, x3, y3);
-    CGPathAddLineToPoint(outline, NULL, x4, y4);
+    CGRect rect = (CGRect) {{ 0.5f*g_window_manager.border_width, 0.5f*g_window_manager.border_width }, { frame.size.width - g_window_manager.border_width, frame.size.height - g_window_manager.border_width }};
+    CGRect fill = CGRectInset(rect, 0.5f*g_window_manager.border_width, 0.5f*g_window_manager.border_width);
+    CGRect clip = { { rect.origin.x + clip_x, rect.origin.y + clip_y }, { rect.size.width + clip_w, rect.size.height + clip_h } };
+    CGPathRef path = CGPathCreateWithRoundedRect(rect, cgrect_clamp_x_radius(rect, g_window_manager.border_radius), cgrect_clamp_y_radius(rect, g_window_manager.border_radius), NULL);
 
     SLSDisableUpdate(g_connection);
     SLSOrderWindow(g_connection, node->feedback_window.id, 0, node->window_order[0]);
     SLSSetWindowShape(g_connection, node->feedback_window.id, 0.0f, 0.0f, frame_region);
     CGContextClearRect(node->feedback_window.context, frame);
+    CGContextClipToRect(node->feedback_window.context, clip);
     CGContextFillRect(node->feedback_window.context, fill);
-    CGContextAddPath(node->feedback_window.context, outline);
+    CGContextAddPath(node->feedback_window.context, path);
     CGContextStrokePath(node->feedback_window.context);
+    CGContextResetClip(node->feedback_window.context);
     CGContextFlush(node->feedback_window.context);
     SLSOrderWindow(g_connection, node->feedback_window.id, 1, node->window_order[0]);
     SLSReenableUpdate(g_connection);
-    CGPathRelease(outline);
+    CGPathRelease(path);
     CFRelease(frame_region);
 }
 
@@ -110,11 +108,6 @@ void insert_feedback_destroy(struct window_node *node)
     }
 }
 
-static inline CGPoint area_center(struct area a)
-{
-    return (CGPoint) { a.x + a.w*0.5f, a.y + a.h*0.5f };
-}
-
 static inline struct area area_from_cgrect(CGRect rect)
 {
     return (struct area) { rect.origin.x, rect.origin.y, rect.size.width, rect.size.height };
@@ -127,7 +120,9 @@ static inline enum window_node_child window_node_get_child(struct window_node *n
 
 static inline enum window_node_split window_node_get_split(struct window_node *node)
 {
-    return node->split != SPLIT_NONE ? node->split : node->area.w / node->area.h >= 1.1618f ? SPLIT_Y : SPLIT_X;
+    if (node->split                != SPLIT_NONE) return node->split;
+    if (g_space_manager.split_type != SPLIT_AUTO) return g_space_manager.split_type;
+    return node->area.w >= node->area.h ? SPLIT_Y : SPLIT_X;
 }
 
 static inline float window_node_get_ratio(struct window_node *node)
@@ -140,33 +135,47 @@ static inline float window_node_get_gap(struct view *view)
     return view->enable_gap ? view->window_gap*0.5f : 0.0f;
 }
 
-static void area_make_pair(struct view *view, struct window_node *node)
+#define area_ax_truncate(a)  \
+    a->x = (int)(a->x + 0.5f); \
+    a->y = (int)(a->y + 0.5f); \
+    a->w = (int)(a->w + 0.5f); \
+    a->h = (int)(a->h + 0.5f);
+
+static void area_make_pair(enum window_node_split split, float gap, float ratio, struct area *parent_area, struct area *left_area, struct area *right_area)
+{
+    if (split == SPLIT_Y) {
+        *left_area = *parent_area;
+        left_area->w *= ratio;
+        left_area->w -= gap;
+
+        *right_area = *parent_area;
+        right_area->x += (parent_area->w * ratio);
+        right_area->w *= (1 - ratio);
+        right_area->x += gap;
+        right_area->w -= gap;
+    } else {
+        *left_area = *parent_area;
+        left_area->h *= ratio;
+        left_area->h -= gap;
+
+        *right_area = *parent_area;
+        right_area->y += (parent_area->h * ratio);
+        right_area->h *= (1 - ratio);
+        right_area->y += gap;
+        right_area->h -= gap;
+    }
+
+    area_ax_truncate(left_area);
+    area_ax_truncate(right_area);
+}
+
+static void area_make_pair_for_node(struct view *view, struct window_node *node)
 {
     enum window_node_split split = window_node_get_split(node);
     float ratio = window_node_get_ratio(node);
     float gap   = window_node_get_gap(view);
 
-    if (split == SPLIT_Y) {
-        node->left->area = node->area;
-        node->left->area.w *= ratio;
-        node->left->area.w -= gap;
-
-        node->right->area = node->area;
-        node->right->area.x += (node->area.w * ratio);
-        node->right->area.w *= (1 - ratio);
-        node->right->area.x += gap;
-        node->right->area.w -= gap;
-    } else {
-        node->left->area = node->area;
-        node->left->area.h *= ratio;
-        node->left->area.h -= gap;
-
-        node->right->area = node->area;
-        node->right->area.y += (node->area.h * ratio);
-        node->right->area.h *= (1 - ratio);
-        node->right->area.y += gap;
-        node->right->area.h -= gap;
-    }
+    area_make_pair(split, gap, ratio, &node->area, &node->left->area, &node->right->area);
 
     node->split = split;
     node->ratio = ratio;
@@ -202,7 +211,7 @@ static inline struct equalize_node equalize_node_add(struct equalize_node a, str
     return (struct equalize_node) { a.y_count + b.y_count, a.x_count + b.x_count, };
 }
 
-static struct equalize_node window_node_equalize(struct window_node *node)
+static struct equalize_node window_node_equalize(struct window_node *node, uint32_t axis_flag)
 {
     if (window_node_is_leaf(node)) {
         return (struct equalize_node) {
@@ -211,16 +220,22 @@ static struct equalize_node window_node_equalize(struct window_node *node)
         };
     }
 
-    struct equalize_node left_leafs  = window_node_equalize(node->left);
-    struct equalize_node right_leafs = window_node_equalize(node->right);
+    struct equalize_node left_leafs  = window_node_equalize(node->left, axis_flag);
+    struct equalize_node right_leafs = window_node_equalize(node->right, axis_flag);
     struct equalize_node total_leafs = equalize_node_add(left_leafs, right_leafs);
 
-    if (node->split == SPLIT_Y) {
-        node->ratio = (float) left_leafs.y_count / total_leafs.y_count;
-        --total_leafs.y_count;
-    } else if (node->split == SPLIT_X) {
-        node->ratio = (float) left_leafs.x_count / total_leafs.x_count;
-        --total_leafs.x_count;
+    if (axis_flag & SPLIT_Y) {
+        if (node->split == SPLIT_Y) {
+            node->ratio = (float) left_leafs.y_count / total_leafs.y_count;
+            --total_leafs.y_count;
+        }
+    }
+
+    if (axis_flag & SPLIT_X) {
+        if (node->split == SPLIT_X) {
+            node->ratio = (float) left_leafs.x_count / total_leafs.x_count;
+            --total_leafs.x_count;
+        }
     }
 
     if (node->parent) {
@@ -239,10 +254,18 @@ static void window_node_split(struct view *view, struct window_node *node, struc
     struct window_node *right = malloc(sizeof(struct window_node));
     memset(right, 0, sizeof(struct window_node));
 
+    struct window_node *zoom = !node->zoom
+                             ? NULL
+                             : node->zoom == node->parent
+                             ? node
+                             : view->root;
+
     if (window_node_get_child(node) == CHILD_SECOND) {
         memcpy(left->window_list, node->window_list, sizeof(uint32_t) * node->window_count);
         memcpy(left->window_order, node->window_order, sizeof(uint32_t) * node->window_count);
         left->window_count = node->window_count;
+        left->zoom = zoom;
+
         right->window_list[0] = window->id;
         right->window_order[0] = window->id;
         right->window_count = 1;
@@ -250,6 +273,8 @@ static void window_node_split(struct view *view, struct window_node *node, struc
         memcpy(right->window_list, node->window_list, sizeof(uint32_t) * node->window_count);
         memcpy(right->window_order, node->window_order, sizeof(uint32_t) * node->window_count);
         right->window_count = node->window_count;
+        right->zoom = zoom;
+
         left->window_list[0] = window->id;
         left->window_order[0] = window->id;
         left->window_count = 1;
@@ -263,13 +288,13 @@ static void window_node_split(struct view *view, struct window_node *node, struc
     node->right = right;
     node->zoom  = NULL;
 
-    area_make_pair(view, node);
+    area_make_pair_for_node(view, node);
 }
 
 void window_node_update(struct view *view, struct window_node *node)
 {
     if (window_node_is_intermediate(node)) {
-        area_make_pair(view, node->parent);
+        area_make_pair_for_node(view, node->parent);
     }
 
     if (window_node_is_leaf(node)) {
@@ -293,37 +318,37 @@ static void window_node_destroy(struct window_node *node)
     free(node);
 }
 
-static void window_node_clear_zoom(struct window_node *node)
-{
-    node->zoom = NULL;
-
-    if (!window_node_is_leaf(node)) {
-        window_node_clear_zoom(node->left);
-        window_node_clear_zoom(node->right);
-    }
-}
-
-void window_node_flush(struct window_node *node)
+void window_node_capture_windows(struct window_node *node, struct window_capture **window_list)
 {
     if (window_node_is_occupied(node)) {
         for (int i = 0; i < node->window_count; ++i) {
             struct window *window = window_manager_find_window(&g_window_manager, node->window_list[i]);
             if (window) {
-                if (node->zoom) {
-                    window_manager_set_window_frame(window, node->zoom->area.x, node->zoom->area.y, node->zoom->area.w, node->zoom->area.h);
-                } else {
-                    window_manager_set_window_frame(window, node->area.x, node->area.y, node->area.w, node->area.h);
+                struct area area = node->zoom ? node->zoom->area : node->area;
+                if (window->border.id) {
+                    area.x += g_window_manager.border_width;
+                    area.y += g_window_manager.border_width;
+                    area.w -= g_window_manager.border_width * 2.0f;
+                    area.h -= g_window_manager.border_width * 2.0f;
                 }
+                ts_buf_push(*window_list, ((struct window_capture) { .window = window, .x = area.x, .y = area.y, .w = area.w, .h = area.h }));
             }
         }
     }
 
     if (!window_node_is_leaf(node)) {
-        window_node_flush(node->left);
-        window_node_flush(node->right);
+        window_node_capture_windows(node->left, window_list);
+        window_node_capture_windows(node->right, window_list);
     }
 }
 
+void window_node_flush(struct window_node *node)
+{
+    struct window_capture *window_list = NULL;
+    window_node_capture_windows(node, &window_list);
+    if (window_list) window_manager_animate_window_list(window_list, ts_buf_len(window_list));
+}
+
 bool window_node_contains_window(struct window_node *node, uint32_t window_id)
 {
     for (int i = 0; i < node->window_count; ++i) {
@@ -394,7 +419,7 @@ struct window_node *window_node_find_prev_leaf(struct window_node *node)
         return node->parent->left;
     }
 
-    return window_node_find_first_leaf(node->parent->left->right);
+    return window_node_find_last_leaf(node->parent->left->right);
 }
 
 struct window_node *window_node_find_next_leaf(struct window_node *node)
@@ -487,43 +512,73 @@ struct window_node *view_find_min_depth_leaf_node(struct window_node *node)
     return NULL;
 }
 
+static inline bool area_is_in_direction(struct area *r1, CGPoint r1_max, struct area *r2, CGPoint r2_max, int direction)
+{
+    if (direction == DIR_NORTH && r1_max.y <= r2->y) return false;
+    if (direction == DIR_EAST  && r2_max.x <= r1->x) return false;
+    if (direction == DIR_SOUTH && r2_max.y <= r1->y) return false;
+    if (direction == DIR_WEST  && r1_max.x <= r2->x) return false;
+
+    if (direction == DIR_NORTH || direction == DIR_SOUTH) {
+        return ((r2_max.x >  r1->x && r2_max.x <= r1_max.x) ||
+                (r2->x    <  r1->x && r2_max.x >  r1_max.x) ||
+                (r2->x    >= r1->x && r2->x    <  r1_max.x));
+    }
+
+    if (direction == DIR_EAST || direction == DIR_WEST) {
+        return ((r2_max.y >  r1->y && r2_max.y <= r1_max.y) ||
+                (r2->y    <  r1->y && r2_max.y >  r1_max.y) ||
+                (r2->y    >= r1->y && r2->y    <  r1_max.y));
+    }
+
+    return false;
+}
+
+static inline int area_distance_in_direction(struct area *r1, CGPoint r1_max, struct area *r2, CGPoint r2_max, int direction)
+{
+    switch (direction) {
+    case DIR_NORTH: {
+        return r2_max.y > r1->y ? r2_max.y - r1->y : r1->y - r2_max.y;
+    } break;
+    case DIR_EAST: {
+        return r2->x < r1_max.x ? r1_max.x - r2->x : r2->x - r1_max.x;
+    } break;
+    case DIR_SOUTH: {
+        return r2->y < r1_max.y ? r1_max.y - r2->y : r2->y - r1_max.y;
+    } break;
+    case DIR_WEST: {
+        return r2_max.x > r1->x ? r2_max.x - r1->x : r1->x - r2_max.x;
+    } break;
+    }
+
+    return INT_MAX;
+}
+
 struct window_node *view_find_window_node_in_direction(struct view *view, struct window_node *source, int direction)
 {
+    int window_count;
+    uint32_t *window_list = space_window_list(view->sid, &window_count, false);
+    if (!window_list) return NULL;
+
     int best_distance = INT_MAX;
+    int best_rank = INT_MAX;
     struct window_node *best_node = NULL;
-    CGPoint source_point = area_center(source->area);
+
+    CGPoint source_area_max = { source->area.x + source->area.w, source->area.y + source->area.h };
 
     struct window_node *target = window_node_find_first_leaf(view->root);
     while (target) {
-        CGPoint target_point = area_center(target->area);
-        int distance = euclidean_distance(source_point, target_point);
-        if (distance >= best_distance) goto next;
+        if (source == target) goto next;
 
-        switch (direction) {
-        case DIR_EAST: {
-            if (target->area.x >= source->area.x + source->area.w) {
+        CGPoint target_area_max = { target->area.x + target->area.w, target->area.y + target->area.h };
+        if (area_is_in_direction(&source->area, source_area_max, &target->area, target_area_max, direction)) {
+            int distance = area_distance_in_direction(&source->area, source_area_max, &target->area, target_area_max, direction);
+            int rank = window_manager_find_rank_of_window_in_list(target->window_order[0], window_list, window_count);
+            if ((distance < best_distance) || (distance == best_distance && rank < best_rank)) {
                 best_node = target;
                 best_distance = distance;
+                best_rank = rank;
             }
-        } break;
-        case DIR_SOUTH: {
-            if (target->area.y >= source->area.y + source->area.h) {
-                best_node = target;
-                best_distance = distance;
-            }
-        } break;
-        case DIR_WEST: {
-            if (target->area.x + target->area.w <= source->area.x) {
-                best_node = target;
-                best_distance = distance;
-            }
-        } break;
-        case DIR_NORTH: {
-            if (target->area.y + target->area.h <= source->area.y) {
-                best_node = target;
-                best_distance = distance;
-            }
-        } break;
         }
 
 next:
@@ -544,10 +599,10 @@ struct window_node *view_find_window_node(struct view *view, uint32_t window_id)
     return NULL;
 }
 
-void view_remove_window_node(struct view *view, struct window *window)
+struct window_node *view_remove_window_node(struct view *view, struct window *window)
 {
     struct window_node *node = view_find_window_node(view, window->id);
-    if (!node) return;
+    if (!node) return NULL;
 
     if (node->window_count > 1) {
         bool removed_entry = false;
@@ -569,17 +624,19 @@ void view_remove_window_node(struct view *view, struct window *window)
         assert(removed_order);
         --node->window_count;
 
-        return;
+        if (view->insertion_point == window->id) {
+            view->insertion_point = node->window_order[0];
+        }
+
+        return NULL;
     }
 
     if (node == view->root) {
-        node->window_count = 0;
-        insert_feedback_destroy(node);
-        node->split = SPLIT_NONE;
-        node->child = CHILD_NONE;
-        node->insert_dir = 0;
         view->insertion_point = 0;
-        return;
+        insert_feedback_destroy(node);
+        memset(node, 0, sizeof(struct window_node));
+        view_update(view);
+        return NULL;
     }
 
     struct window_node *parent = node->parent;
@@ -594,7 +651,11 @@ void view_remove_window_node(struct view *view, struct window *window)
 
     parent->left      = NULL;
     parent->right     = NULL;
-    parent->zoom      = NULL;
+    parent->zoom      = !child->zoom
+                      ? NULL
+                      : child->zoom == parent
+                      ? parent->parent
+                      : view->root;
 
     if (child->insert_dir) {
         parent->feedback_window = child->feedback_window;
@@ -605,16 +666,24 @@ void view_remove_window_node(struct view *view, struct window *window)
     }
 
     if (window_node_is_intermediate(child) && !window_node_is_leaf(child)) {
-        parent->left = child->left;
-        parent->left->parent = parent;
-        parent->left->zoom = NULL;
+        parent->left          = child->left;
+        parent->left->parent  = parent;
+        parent->left->zoom    = !child->left->zoom
+                              ? NULL
+                              : child->left->zoom == child
+                              ? parent
+                              : view->root;
 
-        parent->right = child->right;
+        parent->right         = child->right;
         parent->right->parent = parent;
-        parent->right->zoom = NULL;
+        parent->right->zoom   = !child->right->zoom
+                              ? NULL
+                              : child->right->zoom == child
+                              ? parent
+                              : view->root;
 
-        window_node_clear_zoom(parent);
-        window_node_update(view, parent);
+        window_node_update(view, parent->left);
+        window_node_update(view, parent->right);
     }
 
     insert_feedback_destroy(node);
@@ -623,37 +692,42 @@ void view_remove_window_node(struct view *view, struct window *window)
     free(node);
 
     if (g_space_manager.auto_balance) {
-        window_node_equalize(view->root);
+        window_node_equalize(view->root, SPLIT_X | SPLIT_Y);
         view_update(view);
+        return view->root;
     }
+
+    return parent;
 }
 
 void view_stack_window_node(struct view *view, struct window_node *node, struct window *window)
 {
-    if (node->zoom) {
-        window_manager_set_window_frame(window, node->zoom->area.x, node->zoom->area.y, node->zoom->area.w, node->zoom->area.h);
-    } else {
-        window_manager_set_window_frame(window, node->area.x, node->area.y, node->area.w, node->area.h);
-    }
-
-    node->window_list[node->window_count] = window->id;
-    node->window_order[node->window_count] = window->id;
+    node->window_list[node->window_count]  = window->id;
+    memmove(node->window_order + 1, node->window_order, sizeof(uint32_t) * node->window_count);
+    node->window_order[0] = window->id;
     ++node->window_count;
 }
 
-void view_add_window_node(struct view *view, struct window *window)
+struct window_node *view_add_window_node_with_insertion_point(struct view *view, struct window *window, uint32_t insertion_point)
 {
     if (!window_node_is_occupied(view->root) &&
         window_node_is_leaf(view->root)) {
         view->root->window_list[0] = window->id;
         view->root->window_order[0] = window->id;
         view->root->window_count = 1;
+        return view->root;
     } else if (view->layout == VIEW_BSP) {
+        uint32_t prev_insertion_point = 0;
         struct window_node *leaf = NULL;
 
+        if (insertion_point) {
+            prev_insertion_point = view->insertion_point;
+            view->insertion_point = insertion_point;
+        }
+
         if (view->insertion_point) {
             leaf = view_find_window_node(view, view->insertion_point);
-            view->insertion_point = 0;
+            view->insertion_point = prev_insertion_point;
 
             if (leaf) {
                 bool do_stack = leaf->insert_dir == STACK;
@@ -663,7 +737,7 @@ void view_add_window_node(struct view *view, struct window *window)
 
                 if (do_stack) {
                     view_stack_window_node(view, leaf, window);
-                    return;
+                    return leaf;
                 }
             }
         }
@@ -674,12 +748,23 @@ void view_add_window_node(struct view *view, struct window *window)
         window_node_split(view, leaf, window);
 
         if (g_space_manager.auto_balance) {
-            window_node_equalize(view->root);
+            window_node_equalize(view->root, SPLIT_X | SPLIT_Y);
             view_update(view);
+            return view->root;
         }
+
+        return leaf;
     } else if (view->layout == VIEW_STACK) {
         view_stack_window_node(view, view->root, window);
+        return view->root;
     }
+
+    return NULL;
+}
+
+struct window_node *view_add_window_node(struct view *view, struct window *window)
+{
+    return view_add_window_node_with_insertion_point(view, window, 0);
 }
 
 uint32_t *view_find_window_list(struct view *view, int *window_count)
@@ -687,7 +772,7 @@ uint32_t *view_find_window_list(struct view *view, int *window_count)
     *window_count = 0;
 
     int capacity = 13;
-    uint32_t *window_list = ts_alloc(sizeof(uint32_t) * capacity);
+    uint32_t *window_list = ts_alloc_aligned(sizeof(uint32_t), capacity);
 
     struct window_node *node = window_node_find_first_leaf(view->root);
     while (node) {
@@ -724,29 +809,20 @@ void view_flush(struct view *view)
 
 void view_serialize(FILE *rsp, struct view *view)
 {
+    char *uuid = ts_cfstring_copy(view->suuid);
     int buffer_size = MAXLEN;
     size_t bytes_written = 0;
     char buffer[MAXLEN] = {};
     char *cursor = buffer;
 
-    int count = 0;
-    uint32_t windows[MAXLEN] = {};
-
     int window_count = 0;
     uint32_t *window_list = space_window_list(view->sid, &window_count, true);
-    if (window_list) {
-        for (int i = 0; i < window_count; ++i) {
-            if (window_manager_find_window(&g_window_manager, window_list[i])) {
-                windows[count++] = window_list[i];
-            }
-        }
-    }
 
-    for (int i = 0; i < count; ++i) {
-        if (i < count - 1) {
-            bytes_written = snprintf(cursor, buffer_size, "%d, ", windows[i]);
+    for (int i = 0; i < window_count; ++i) {
+        if (i < window_count - 1) {
+            bytes_written = snprintf(cursor, buffer_size, "%d, ", window_list[i]);
         } else {
-            bytes_written = snprintf(cursor, buffer_size, "%d", windows[i]);
+            bytes_written = snprintf(cursor, buffer_size, "%d", window_list[i]);
         }
 
         cursor += bytes_written;
@@ -761,28 +837,30 @@ void view_serialize(FILE *rsp, struct view *view)
     fprintf(rsp,
             "{\n"
             "\t\"id\":%lld,\n"
-            "\t\"label\":\"%s\",\n"
+            "\t\"uuid\":\"%s\",\n"
             "\t\"index\":%d,\n"
+            "\t\"label\":\"%s\",\n"
+            "\t\"type\":\"%s\",\n"
             "\t\"display\":%d,\n"
             "\t\"windows\":[%s],\n"
-            "\t\"type\":\"%s\",\n"
-            "\t\"visible\":%d,\n"
-            "\t\"focused\":%d,\n"
-            "\t\"native-fullscreen\":%d,\n"
             "\t\"first-window\":%d,\n"
-            "\t\"last-window\":%d\n"
+            "\t\"last-window\":%d,\n"
+            "\t\"has-focus\":%s,\n"
+            "\t\"is-visible\":%s,\n"
+            "\t\"is-native-fullscreen\":%s\n"
             "}",
             view->sid,
-            space_label ? space_label->label : "",
+            uuid ? uuid : "<unknown>",
             space_manager_mission_control_index(view->sid),
+            space_label ? space_label->label : "",
+            view_type_str[view->layout],
             display_arrangement(space_display_id(view->sid)),
             buffer,
-            view_type_str[view->layout],
-            space_is_visible(view->sid),
-            view->sid == g_space_manager.current_space_id,
-            space_is_fullscreen(view->sid),
             first_leaf ? first_leaf->window_order[0] : 0,
-            last_leaf ? last_leaf->window_order[0] : 0);
+            last_leaf ? last_leaf->window_order[0] : 0,
+            json_bool(view->sid == g_space_manager.current_space_id),
+            json_bool(space_is_visible(view->sid)),
+            json_bool(space_is_fullscreen(view->sid)));
 }
 
 void view_update(struct view *view)
@@ -814,7 +892,7 @@ struct view *view_create(uint64_t sid)
     view->enable_padding = true;
     view->enable_gap = true;
     view->sid = sid;
-    view->suuid = space_uuid(sid);
+    view->suuid = SLSSpaceCopyName(g_connection, sid);
 
     if (space_is_user(view->sid)) {
         if (!view->custom_layout)         view->layout         = g_space_manager.layout;
