@@ -1,5 +1,4 @@
 #define SA_SOCKET_PATH_FMT      "/tmp/yabai-sa_%s.socket"
-
 #define SOCKET_PATH_FMT         "/tmp/yabai_%s.socket"
 #define LCFILE_PATH_FMT         "/tmp/yabai_%s.lock"
 
@@ -13,18 +12,12 @@
 #define CONFIG_OPT_LONG         "--config"
 #define CONFIG_OPT_SHRT         "-c"
 
-#define SCRPT_ADD_INSTALL_OPT   "--install-sa"
 #define SCRPT_ADD_UNINSTALL_OPT "--uninstall-sa"
-#define SCRPT_ADD_CHECK_OPT     "--check-sa"
 #define SCRPT_ADD_LOAD_OPT      "--load-sa"
 
-#define MAJOR  3
-#define MINOR  3
-#define PATCH 10
-
-#define CONNECTION_CALLBACK(name) void name(uint32_t type, void *data, size_t data_length, void *context, int cid)
-typedef CONNECTION_CALLBACK(connection_callback);
-extern CGError SLSRegisterConnectionNotifyProc(int cid, connection_callback *handler, uint32_t event, void *context);
+#define MAJOR  5
+#define MINOR  0
+#define PATCH  2
 
 struct event_loop g_event_loop;
 void *g_workspace_context;
@@ -34,13 +27,13 @@ struct space_manager g_space_manager;
 struct window_manager g_window_manager;
 struct mouse_state g_mouse_state;
 struct event_tap g_event_tap;
-struct daemon g_daemon;
 int g_normal_window_level;
 int g_floating_window_level;
 int g_connection;
 
-struct signal *g_signal_event[EVENT_TYPE_COUNT];
-bool g_mission_control_active;
+struct signal *g_signal_event[SIGNAL_TYPE_COUNT];
+struct memory_pool g_signal_storage;
+int g_mission_control_active;
 char g_sa_socket_file[MAXLEN];
 char g_socket_file[MAXLEN];
 char g_config_file[4096];
@@ -58,14 +51,6 @@ static int client_send_message(int argc, char **argv)
         error("yabai-msg: 'env USER' not set! abort..\n");
     }
 
-    int sockfd;
-    char socket_file[MAXLEN];
-    snprintf(socket_file, sizeof(socket_file), SOCKET_PATH_FMT, user);
-
-    if (!socket_connect_un(&sockfd, socket_file)) {
-        error("yabai-msg: failed to connect to socket..\n");
-    }
-
     int message_length = argc;
     int argl[argc];
 
@@ -74,9 +59,10 @@ static int client_send_message(int argc, char **argv)
         message_length += argl[i];
     }
 
-    char message[message_length];
-    char *temp = message;
+    char *message = malloc(sizeof(int)+message_length);
+    char *temp = sizeof(int)+message;
 
+    memcpy(message, &message_length, sizeof(int));
     for (int i = 1; i < argc; ++i) {
         memcpy(temp, argv[i], argl[i]);
         temp += argl[i];
@@ -84,36 +70,41 @@ static int client_send_message(int argc, char **argv)
     }
     *temp++ = '\0';
 
-    if (!socket_write_bytes(sockfd, message, message_length)) {
+    int sockfd;
+    char socket_file[MAXLEN];
+    snprintf(socket_file, sizeof(socket_file), SOCKET_PATH_FMT, user);
+
+    if (!socket_open(&sockfd)) {
+        error("yabai-msg: failed to open socket..\n");
+    }
+
+    if (!socket_connect(sockfd, socket_file)) {
+        error("yabai-msg: failed to connect to socket..\n");
+    }
+
+    if (send(sockfd, message, sizeof(int)+message_length, 0) == -1) {
         error("yabai-msg: failed to send data..\n");
     }
 
     shutdown(sockfd, SHUT_WR);
+    free(message);
 
     int result = EXIT_SUCCESS;
-    int byte_count = 0;
+    FILE *output = stdout;
+    int bytes_read = 0;
     char rsp[BUFSIZ];
 
-    struct pollfd fds[] = {
-        { sockfd, POLLIN, 0 }
-    };
+    while ((bytes_read = read(sockfd, rsp, sizeof(rsp)-1)) > 0) {
+        rsp[bytes_read] = '\0';
 
-    while (poll(fds, 1, -1) > 0) {
-        if (fds[0].revents & POLLIN) {
-            if ((byte_count = recv(sockfd, rsp, sizeof(rsp)-1, 0)) <= 0) {
-                break;
-            }
-
-            rsp[byte_count] = '\0';
-
-            if (rsp[0] == FAILURE_MESSAGE[0]) {
-                result = EXIT_FAILURE;
-                fprintf(stderr, "%s", rsp + 1);
-                fflush(stderr);
-            } else {
-                fprintf(stdout, "%s", rsp);
-                fflush(stdout);
-            }
+        if (rsp[0] == FAILURE_MESSAGE[0]) {
+            result = EXIT_FAILURE;
+            output = stderr;
+            fprintf(output, "%s", rsp + 1);
+            fflush(output);
+        } else {
+            fprintf(output, "%s", rsp);
+            fflush(output);
         }
     }
 
@@ -176,9 +167,12 @@ static void exec_config_file(void)
         return;
     }
 
-    if (!fork_exec(g_config_file, NULL)) {
+    int pid = fork();
+    if (pid == 0) {
+        char *exec[] = { "/usr/bin/env", "sh", "-c", g_config_file, NULL};
+        exit(execvp(exec[0], exec));
+    } else if (pid == -1) {
         notify("configuration", "failed to execute file '%s'", g_config_file);
-        return;
     }
 }
 
@@ -203,12 +197,30 @@ static inline void init_misc_settings(void)
     g_connection = SLSMainConnectionID();
     g_normal_window_level   = CGWindowLevelForKey(LAYER_NORMAL);
     g_floating_window_level = CGWindowLevelForKey(LAYER_ABOVE);
+
+#if 0
+    hook_nsobject_autorelease();
+    hook_autoreleasepool_drain();
+    hook_autoreleasepool_release();
+#endif
 }
 #pragma clang diagnostic pop
 
 static CONNECTION_CALLBACK(connection_handler)
 {
-    event_loop_post(&g_event_loop, MISSION_CONTROL_ENTER, NULL, 0, NULL);
+    if (type == 1204) {
+        event_loop_post(&g_event_loop, MISSION_CONTROL_ENTER, NULL, 0, NULL);
+    } else if (type == 806) {
+        event_loop_post(&g_event_loop, SLS_WINDOW_MOVED, (void *) (intptr_t) (*(uint32_t *) data), 0, NULL);
+    } else if (type == 807) {
+        event_loop_post(&g_event_loop, SLS_WINDOW_RESIZED, (void *) (intptr_t) (* (uint32_t *) data), 0, NULL);
+    } else if (type == 808) {
+        event_loop_post(&g_event_loop, SLS_WINDOW_ORDER_CHANGED, (void *) (intptr_t) (* (uint32_t *) data), 0, NULL);
+    } else if (type == 815) {
+        event_loop_post(&g_event_loop, SLS_WINDOW_IS_VISIBLE, (void *) (intptr_t) (* (uint32_t *) data), 0, NULL);
+    } else if (type == 816) {
+        event_loop_post(&g_event_loop, SLS_WINDOW_IS_INVISIBLE, (void *) (intptr_t) (* (uint32_t *) data), 0, NULL);
+    }
 }
 
 static void parse_arguments(int argc, char **argv)
@@ -224,18 +236,10 @@ static void parse_arguments(int argc, char **argv)
         exit(client_send_message(argc-1, argv+1));
     }
 
-    if (string_equals(argv[1], SCRPT_ADD_INSTALL_OPT)) {
-        exit(scripting_addition_install());
-    }
-
     if (string_equals(argv[1], SCRPT_ADD_UNINSTALL_OPT)) {
         exit(scripting_addition_uninstall());
     }
 
-    if (string_equals(argv[1], SCRPT_ADD_CHECK_OPT)) {
-        exit(scripting_addition_check());
-    }
-
     if (string_equals(argv[1], SCRPT_ADD_LOAD_OPT)) {
         exit(scripting_addition_load());
     }
@@ -282,7 +286,11 @@ int main(int argc, char **argv)
         error("yabai: could not initialize event_loop! abort..\n");
     }
 
-    if (!ts_init(KILOBYTES(128))) {
+    if (!memory_pool_init(&g_signal_storage, KILOBYTES(128))) {
+        error("yabai: could not allocate memory for event_signal! abort..\n");
+    }
+
+    if (!ts_init(MEGABYTES(4))) {
         error("yabai: could not allocate temporary storage! abort..\n");
     }
 
@@ -299,21 +307,32 @@ int main(int argc, char **argv)
     process_manager_begin(&g_process_manager);
     workspace_event_handler_begin(&g_workspace_context);
     event_tap_begin(&g_event_tap, EVENT_MASK_MOUSE, mouse_handler);
-    SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1204, NULL);
 
-    if (!socket_daemon_begin_un(&g_daemon, g_socket_file, message_handler)) {
-        error("yabai: could not initialize daemon! abort..\n");
+    if (workspace_is_macos_monterey() || workspace_is_macos_ventura()) {
+        mission_control_observe();
+    } else {
+        SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1204, NULL);
     }
 
-    if (!workspace_is_macos_bigsur()) {
-        if (scripting_addition_is_installed()) {
-            scripting_addition_load();
-        } else {
-            notify("scripting-addition", "payload is not installed, some features will not work!");
-        }
+    SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 806, NULL);
+    SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 807, NULL);
+    SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 808, NULL);
+    SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 815, NULL);
+    SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 816, NULL);
+
+    if (!message_loop_begin(g_socket_file)) {
+        error("yabai: could not initialize message_loop! abort..\n");
     }
 
     exec_config_file();
-    CFRunLoopRun();
+
+    for (;;) {
+        NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+        CFRunLoopRunResult result = CFRunLoopRunInMode(kCFRunLoopDefaultMode, 300, true);
+        [pool drain];
+
+        if (result == kCFRunLoopRunFinished || result == kCFRunLoopRunStopped) break;
+    }
+
     return 0;
 }
